using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

using TIS.Imaging;
using System.Runtime.InteropServices;

namespace SoftTrigger_Stereo
{
    public partial class Form1 : Form
    {
        private VCDButtonProperty SoftTriggerLeft;
        private VCDButtonProperty SoftTriggerRight;
        private bool NewImageLeft = false;
        private bool NewImageRight = false;
        
        public Form1()
        {
            InitializeComponent();
            chkContinuousTrigger.Enabled = false;
            btnTrigger.Enabled = false;
            
            //Setup the left and right IC Imaging Control
            SetupIC(icLeft, "Left", out SoftTriggerLeft);
            SetupIC(icRight, "Right", out SoftTriggerRight);
        }

        /// <summary>
        /// Setup the IC Imaging Control instances
        /// </summary>
        /// <param name="CurrentIC"></param>
        /// <param name="Position"></param>
        /// <param name="SoftTrigger"></param>
        private void SetupIC(ICImagingControl CurrentIC, String Position,out VCDButtonProperty  SoftTrigger)
        {
            CurrentIC.LiveDisplayDefault = false;
            CurrentIC.LiveDisplaySize = CurrentIC.Size;
            CurrentIC.LiveCaptureContinuous = true;
            CurrentIC.MemoryCurrentGrabberColorformat = ICImagingControlColorformats.ICRGB24;
            

            SoftTrigger = null;
            try
            {
                // Try to load the previously used video capture device.
                CurrentIC.LoadDeviceStateFromFile(Position + ".xml", true);
            }
            catch { }

            if (CurrentIC.DeviceValid)
            {
                try
                {
                    // Try to load the previously used settings.
                    CurrentIC.LoadDeviceStateFromFile("Settings.xml", false);
                }
                catch { }
                CurrentIC.LiveStart();
                CurrentIC.ImageBuffers[0].Lock();

                // Get the software trigger property of the video capture device.
                SoftTrigger = (VCDButtonProperty)CurrentIC.VCDPropertyItems.FindInterface(VCDIDs.VCDID_TriggerMode + 
                    ":{FDB4003C-552C-4FAA-B87B-42E888D54147}:" +
                    VCDIDs.VCDInterface_Button);
            }
        }

        private void btnDeviceLeft_Click(object sender, EventArgs e)
        {
            SelectDevice(icLeft, "Left", out SoftTriggerLeft);
        }

        private void btnDeviceRight_Click(object sender, EventArgs e)
        {
            SelectDevice(icRight, "Right",out SoftTriggerRight);
        }

        /// <summary>
        /// Load a new video capture for the passed IC Imaging Control instance.
        /// </summary>
        /// <param name="CurrentIC"></param>
        /// <param name="Position"></param>
        private void SelectDevice(ICImagingControl CurrentIC, String Position, out VCDButtonProperty SoftTrigger)
        {
            SoftTrigger = null;
            if (CurrentIC.LiveVideoRunning)
            {
                CurrentIC.LiveStop();
            }

            CurrentIC.ShowDeviceSettingsDialog();

            if (CurrentIC.DeviceValid)
            {
                // Save the device to a file, so it will be loaded next program start automatically.
                CurrentIC.SaveDeviceStateToFile(Position + ".xml");
                CurrentIC.LiveStart();
                CurrentIC.ImageBuffers[0].Lock();
                // Get the software trigger property of the video capture device.
                SoftTrigger = (VCDButtonProperty)CurrentIC.VCDPropertyItems.FindInterface(VCDIDs.VCDID_TriggerMode + ":{FDB4003C-552C-4FAA-B87B-42E888D54147}:" + VCDIDs.VCDInterface_Button);
            }
        }

        /// <summary>
        /// configure the left video capture device. After doing so, set the same settings
        /// in right video capture device.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnSettings_Click(object sender, EventArgs e)
        {
            if (icLeft.DeviceValid)
            {
                // Configure the left video capture device,
                icLeft.ShowPropertyDialog();
                // Save the properties to the "settings.xml" file
                icLeft.SaveDeviceStateToFile("Settings.xml");

                if (icRight.DeviceValid)
                {
                    try
                    {
                        // Now configure the right video capture device in the same way as the left
                        // by loading the setting from the previously saved "settings.xml".
                        icRight.LoadDeviceStateFromFile("Settings.xml", false);
                    }
                    catch { }
                }
            }
        }

        /// <summary>
        /// Handler for the enable trigger checkbox.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void chkEnableTrigger_CheckedChanged(object sender, EventArgs e)
        {
            if (chkEnableTrigger.Checked)
            {
                SetTrigger(icLeft, true);
                SetTrigger(icRight, true);
                btnTrigger.Enabled = true;
                chkContinuousTrigger.Enabled = true;
            }
            else
            {
                timer1.Enabled = false;
                SetTrigger(icLeft, false);
                SetTrigger(icRight, false);
                btnTrigger.Enabled = false;
                chkContinuousTrigger.Enabled = false;
                chkContinuousTrigger.Checked = false;
            }
        }

        /// <summary>
        /// Enable or disable the external trigger property of the passed IC Imaging Control 
        /// instance.
        /// </summary>
        /// <param name="ic"></param>
        /// <param name="OnOff"></param>
        /// <returns></returns>
        public bool SetTrigger(ICImagingControl ic, bool OnOff)
        {

            VCDSwitchProperty Trigger;
            Trigger = (VCDSwitchProperty)ic.VCDPropertyItems.FindInterface(VCDIDs.VCDID_TriggerMode + ":" + VCDIDs.VCDElement_Value + ":" + VCDIDs.VCDInterface_Switch);

            if (Trigger != null)
            {
                Trigger.Switch = OnOff;
                return true;

            }
            return false;
        }

        /// <summary>
        /// Create the software trigger event for both video capture devices.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnTrigger_Click(object sender, EventArgs e)
        {
            GenerateTrigger();
        }

        /// <summary>
        /// Generate the software triggers continuously by using a timer event.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void timer1_Tick(object sender, EventArgs e)
        {
            GenerateTrigger();
        }


        /// <summary>
        /// Generate the trigger pulse, check for both images arrived and create the red/green image
        /// </summary>
        private void GenerateTrigger()
        {
            if (SoftTriggerLeft != null && SoftTriggerRight != null )
            {
                NewImageLeft = false;
                NewImageRight = false;

                // Trigger the software triggers for both video capture devices,
                SoftTriggerLeft.Push();
                SoftTriggerRight.Push();

                int Tries = 5;
                // Use maximum of 5 tries for check of both images have arrived.
                while (Tries > 0 && (NewImageLeft == false || NewImageRight == false))
                {
                    System.Threading.Thread.Sleep(100);
                    Tries--;
                }

                if (NewImageLeft == false || NewImageRight == false)
                {
                    MessageBox.Show("It was not possible to grab both images.\nPlease try again");
                }
                else
                {
                    // Both images are available, now create the red green image.
                    CreateRedGreenImage();
                }
            }
        }
        
        /// <summary>
        /// In the ImageAvailable event handlers, the flags for new images are set to true. 
        /// If both are set to true, the application knows, both images have arrived 
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void icLeft_ImageAvailable(object sender, ICImagingControl.ImageAvailableEventArgs e)
        {
            NewImageLeft = true;
        }

        private void icRight_ImageAvailable(object sender, ICImagingControl.ImageAvailableEventArgs e)
        {
            NewImageRight = true;
        }


        /// <summary>
        /// Create the red / green image in the locked buffer of the icLeft control.
        /// </summary>
        private void CreateRedGreenImage()
        {
            ImageBuffer IB = icLeft.ImageBuffers[0];
            ImageBuffer Left = icLeft.ImageActiveBuffer;
            ImageBuffer Right = icRight.ImageActiveBuffer;
            int BufferSize = IB.Lines * IB.BytesPerLine;
            
            
            for (int i = 0; i < BufferSize; i += 3)
            {
                IB[i] = 0;             // Blue
                IB[i + 1] = Right[i];  // Green, the red glass on the right eye
                IB[i + 2] = Left[i];   // Red,  the green glass on the left eye
            }

            pictureBox1.Image = IB.Bitmap;
            pictureBox1.Update();
        }

        /// <summary>
        /// Enable or disable the timer.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void chkContinuousTrigger_CheckedChanged(object sender, EventArgs e)
        {
            if (chkContinuousTrigger.Checked)
            {
                timer1.Enabled = true;
            }
            else
            {
                timer1.Enabled = false;
            }
        }



    }
}
